unit AddUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, Buttons,
  ListUnit, Recording, GaussUnit;

type
  TFillMode = (fmAdd, fmModify);

  TAddForm = class(TForm)
    StringGrid: TStringGrid;
    edTitle: TEdit;
    btnCancel: TBitBtn;
    btnOK: TBitBtn;
    cbSaveAccurate: TCheckBox;
    procedure cbSaveAccurateClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
  private
    FFillMode: TFillMode;
  public
    procedure FillForm(AFillMode: TFillMode; Dim: integer; Title: string;
      solut: TabFloat);
    procedure FillGrid(Dim: integer; solut: TabFloat);
    procedure FillEquationInfo(equation: TEquation);
    function EmptyResults: boolean;
  end;

var
  AddForm: TAddForm;

implementation

uses
  ComCtrls;
{$R *.dfm}
{ TAddForm }

procedure TAddForm.FillForm(AFillMode: TFillMode; Dim: integer; Title: string;
  solut: TabFloat);
begin
  FFillMode := AFillMode;
  edTitle.Text := Title;
  StringGrid.RowCount := Dim;
  FillGrid(Dim, solut);
end;

procedure TAddForm.FillGrid(Dim: integer; solut: TabFloat);
var
  i: integer;
begin
  StringGrid.ColWidths[0] := 30;
  StringGrid.ColWidths[1] := 200;
  for i := 0 to Dim - 1 do
  begin
    StringGrid.Cells[0, i] := Format('x%d=', [i]);
    StringGrid.Cells[1, i] := '';
  end;
  if solut = nil then
  begin
    cbSaveAccurate.Checked := false;
  end
  else
  begin
    cbSaveAccurate.Checked := true;
    for i := 0 to Length(solut) - 1 do
      StringGrid.Cells[1, i] := FloatToStr(solut[i]);
  end;
end;

procedure TAddForm.cbSaveAccurateClick(Sender: TObject);
begin
  StringGrid.Enabled := cbSaveAccurate.Checked;
  if StringGrid.Enabled then
    StringGrid.Color := clWindow
  else
    StringGrid.Color := clInactiveBorder;
end;

procedure TAddForm.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TAddForm.btnOKClick(Sender: TObject);
var
  equation: TEquation;
  item: TListItem;
  i: integer;
begin
  FormStyle := fsNormal;
  if FFillMode = fmAdd then
  begin
    equation := TEquation.Create;
    equation.Dim := Model.Size;
    item := ListForm.ListView.Items.Add;
    item.Data := equation;
  end
  else
  begin
    item := ListForm.ListView.Selected;
    if item = nil then
      exit;
    equation := item.Data;
    Assert(equation <> nil);
  end;
  FillEquationInfo(equation);
  item.Caption := equation.Title;
  if FFillMode = fmAdd then
    with Model do
    begin
      GaussForm.GridToArrays;
      SetLength(equation.A, Length(A), Length(A[0]));
      for i := 0 to Length(equation.A) - 1 do
        Move(A[i, 0], equation.A[i, 0], Length(A[i]) * SizeOf(A[i, 0]));
      SetLength(equation.B, Length(B));
      Move(B[0], equation.B[0], Length(B) * SizeOf(B[0]));
      item.SubItems.Add(IntToStr(equation.Dim));
      if equation.hasSolutions then
        item.SubItems.Add('results')
      else
        item.SubItems.Add('lack');
    end
    else if FFillMode = fmModify then
    begin
      item.SubItems[0] := IntToStr(equation.Dim);
      if equation.hasSolutions then
        item.SubItems[1] := 'results'
      else
        item.SubItems[1] := 'lack';
    end;
  Close;
end;

procedure TAddForm.FillEquationInfo(equation: TEquation);
var
  i: integer;
begin
  equation.Title := edTitle.Text;
  equation.hasSolutions := cbSaveAccurate.Checked;
  if equation.hasSolutions then
  begin
    SetLength(equation.Solutions, equation.Dim);
    for i := 0 to equation.Dim - 1 do
      if StringGrid.Cells[1, i] = '' then
        equation.Solutions[i] := 0
      else
        equation.Solutions[i] := StrToFloat(StringGrid.Cells[1, i]);
  end
  else
    SetLength(equation.Solutions, 0);
end;

function TAddForm.EmptyResults: boolean;
var
  i: integer;
begin
  result := false;
  for i := 0 to StringGrid.RowCount - 1 do
    if StringGrid.Cells[1, i] <> '' then
      exit;
  result := true;
end;

end.
