unit ListUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, Menus, ImgList, Buttons,
  Recording, GaussUnit;

type
  TEquation = class
    Title: string;
    Dim: integer;
    A: Tab2DFloat;
    b: TabFloat;
    hasSolutions: boolean;
    Solutions: TabFloat;
  end;

  TListForm = class(TForm)
    btnLoad: TButton;
    ListView: TListView;
    btnSave: TButton;
    btnAdd: TButton;
    btnRemove: TButton;
    btnModify: TButton;
    btnMoveDown: TSpeedButton;
    btnMoveUp: TSpeedButton;
    MainMenu: TMainMenu;
    ImageList: TImageList;
    miOrder: TMenuItem;
    miMoveUp: TMenuItem;
    miMoveDown: TMenuItem;
    btnHide: TButton;
    procedure btnLoadClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnModifyClick(Sender: TObject);
    procedure ListViewDblClick(Sender: TObject);
    procedure ListViewChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure btnAddClick(Sender: TObject);
    procedure btnMoveUpClick(Sender: TObject);
    procedure btnMoveDownClick(Sender: TObject);
    procedure miMoveUpClick(Sender: TObject);
    procedure miMoveDownClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure btnHideClick(Sender: TObject);
  private
    EquationList: TList;
    procedure Load(filename: string);
    procedure Save(filename: string);
    procedure clearList;
  public
    Loaded: boolean;
    procedure MoveSelected(Down: boolean);
    procedure Remove;
  end;

var
  ListForm: TListForm;

function pdsep(Str: string): string;

implementation

uses OptionsUnit, AddUnit;
{$R *.dfm}

function DatPath: string;
begin
  result := AppDir() + 'interesting.dat';
end;

procedure TListForm.btnHideClick(Sender: TObject);
begin
  Hide;
end;

procedure TListForm.btnLoadClick(Sender: TObject);
var
  i: integer;
  rownanie: TEquation;
  Item: TListItem;
begin
  ListView.Clear;
  Load(DatPath);
  for i := 0 to EquationList.Count - 1 do
  begin
    rownanie := EquationList[i];
    Item := ListView.Items.Add;
    Item.Data := rownanie;
    Item.Caption := rownanie.Title;
    Item.SubItems.Add(IntToStr(rownanie.Dim));
    if rownanie.hasSolutions then
      Item.SubItems.Add('results')
    else
      Item.SubItems.Add('lack');
  end;
  Loaded := true;
end;

procedure TListForm.clearList;
var
  i: integer;
begin
  for i := 0 to EquationList.Count - 1 do
    TEquation(EquationList[i]).Free;
  EquationList.Clear;
end;

// comma to decimal point or opposite as separator
function pdsep(Str: string): string;
var
  n: integer;
begin
  if DecimalSeparator = '.' then
  begin
    n := Pos(',', Str);
    if n > 0 then
      Str[n] := '.';
  end
  else if DecimalSeparator = ',' then
  begin
    n := Pos('.', Str);
    if n > 0 then
      Str[n] := ',';
  end;
  result := Str;
end;

// comma to decimal point - all occurences
function pdkAll(Str: string): string;
var
  i: integer;
begin
  result := Str;
  for i := 1 to Length(result) do
    if result[i] = ',' then
      result[i] := '.';
end;

function ValFloat(S: PChar; var V: extended; var Len: integer): boolean;
var
  numBuffer: array [0 .. 32] of char;
var
  Code: integer;
begin
  result := true;
  Val(S, V, Code);
  if Code > 0 then
  begin
    Len := Code - 1;
    StrLCopy(numBuffer, S, Len);
    Val(numBuffer, V, Code);
    if Code <> 0 then
      result := false;
  end
  else
    Len := StrLen(S);
end;

procedure TListForm.Save(filename: string);
var
  f: TextFile;
  i, j, k: integer;
  rownanie: TEquation;
begin
  AssignFile(f, filename);
  Rewrite(f);
  for i := 0 to ListView.Items.Count - 1 do
  begin
    rownanie := ListView.Items[i].Data;
    writeln(f, rownanie.Title);
    writeln(f, rownanie.Dim);
    for j := 0 to rownanie.Dim - 1 do
    begin
      for k := 0 to rownanie.Dim - 1 do
        write(f, pdkAll(FloatToStr(rownanie.A[j, k])), #9);
      write(f, pdkAll(FloatToStr(rownanie.b[j])));
      writeln(f);
    end;
    write(f, 'solutions=');
    if rownanie.hasSolutions then
      for j := 0 to rownanie.Dim - 1 do
      begin
        write(f, pdkAll(FloatToStr(rownanie.Solutions[j])));
        if j < rownanie.Dim - 1 then
          write(f, #9);
      end;
    writeln(f);
    writeln(f);
  end;
  CloseFile(f);
end;

procedure TListForm.Load(filename: string);
const
  SIZEBUF = 4 * 1024;
var
  f: TextFile;
  Line: string;
  i, j, numlen: integer;
  equation: TEquation;
  S: PChar;
  d: extended;
  textBuf: pointer;
label LabelClose;
begin
  clearList;
  AssignFile(f, filename);
  GetMem(textBuf, SIZEBUF);
  System.SetTextBuf(f, textBuf^, SIZEBUF); // to tylko aby przyspieszyc wieksze pliki
  Reset(f);
  equation := nil;
  if not Eof(f) then
    repeat
      repeat
        readln(f, Line);
        if Eof(f) then
          goto LabelClose;
      until Trim(Line) <> '';
      equation := TEquation.Create;
      equation.Title := Line;
      readln(f, Line);
      if Eof(f) then
        break;
      equation.Dim := StrToInt(Line);
      SetLength(equation.A, equation.Dim, equation.Dim);
      SetLength(equation.b, equation.Dim);
      for i := 0 to equation.Dim - 1 do
      begin
        readln(f, Line);
        if Eof(f) then
          goto LabelClose;
        Line := pdkAll(Line);
        S := PChar(Line);
        for j := 0 to equation.Dim - 1 do
        begin
          if not ValFloat(S, d, numlen) then
            d := 0;
          equation.A[i, j] := d;
          inc(S, numlen);
          while AnsiChar(S^) in [' ', #9] do
            inc(S);
        end;
        if not ValFloat(S, d, numlen) then
          d := 0;
        equation.b[i] := d;
      end;
      readln(f, Line);
      Line := pdkAll(Line);
      S := PChar(Line);
      S := StrScan(S, '=');
      if S <> nil then
      begin
        inc(S);
        SetLength(equation.Solutions, equation.Dim);
        equation.hasSolutions := true;
        for i := 0 to equation.Dim - 1 do
        begin
          while AnsiChar(S^) in [' ', #9] do
            inc(S);
          if not ValFloat(S, d, numlen) then
          begin
            SetLength(equation.Solutions, 0);
            equation.hasSolutions := false;
          end
          else
            equation.Solutions[i] := d;
          inc(S, numlen);
        end;
      end;
      EquationList.Add(equation);
      equation := nil;
    until Eof(f);
  LabelClose :
    if equation <> nil then
      EquationList.Add(equation);
  CloseFile(f);
  FreeMem(textBuf);
end;

procedure TListForm.FormCreate(Sender: TObject);
begin
  EquationList := TList.Create;
end;

procedure TListForm.FormDestroy(Sender: TObject);
begin
  ListView.OnChange := nil; // kiedy zwolniona lista, nie moze wolac metody gdzie sie do nich odwoluje
  clearList;
  EquationList.Free;
end;

procedure TListForm.btnModifyClick(Sender: TObject);
var
  rownanie: TEquation;
begin
  if ListView.Selected = nil then
    exit;
  rownanie := ListView.Selected.Data;
  Assert(rownanie <> nil);
  with rownanie do
    AddForm.FillForm(fmModify, Dim, Title, Solutions);
  AddForm.FormStyle := fsStayOnTop;
  AddForm.Show;
end;

procedure TListForm.btnAddClick(Sender: TObject);
var
  solut: TabFloat;
begin
  if model.Results.existsExact then
    solut := model.Results.exact
  else if model.Results.is_calculated then
    solut := model.Results.calculated
  else
    solut := nil;
  AddForm.FillForm(fmAdd, model.Size, '', solut);
  AddForm.FormStyle := fsStayOnTop;
  AddForm.Show;
end;

procedure TListForm.ListViewDblClick(Sender: TObject);
begin
  btnModifyClick(Sender);
end;

procedure TListForm.ListViewChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
var
  rownanie: TEquation;
  i, j: integer;
begin
  if not Item.Selected then
    exit;
  rownanie := Item.Data;
  if rownanie = nil then
    exit;
  btnMoveUp.Visible := Item.Index > 0;
  btnMoveDown.Visible := Item.Index < ListView.Items.Count - 1;
  model.Init(Length(rownanie.A), rownanie.A, rownanie.b);
  model.Results.CopyExact(rownanie.Solutions);
  model.Size := rownanie.Dim;
  GaussForm.SetGridSize;
  GaussForm.ArraysToGrid;
  model.Results.existsExact := rownanie.hasSolutions;
  with model do
    for i := 0 to Size do
      for j := 0 to Size + 1 do
        GaussForm.StringGrid.Objects[j, i] := nil;
end;

procedure TListForm.btnMoveUpClick(Sender: TObject);
begin
  MoveSelected(false);
end;

procedure TListForm.btnMoveDownClick(Sender: TObject);
begin
  MoveSelected(true);
end;

procedure TListForm.miMoveUpClick(Sender: TObject);
begin
  MoveSelected(false);
end;

procedure TListForm.miMoveDownClick(Sender: TObject);
begin
  MoveSelected(true);
end;

procedure TListForm.MoveSelected(Down: boolean);
var
  index: integer;
  Item: TListItem;
  Str: array [0 .. 2] of string;
  Data: pointer;
begin
  Item := ListView.Selected;
  if Item = nil then
    exit;
  index := Item.Index;
  if Down then
  begin
    if index >= ListView.Items.Count - 1 then
      exit;
  end
  else if index <= 0 then
    exit;
  Str[0] := Item.Caption;
  Str[1] := Item.SubItems[0];
  Str[2] := Item.SubItems[1];
  Data := Item.Data;
  ListView.Items.Delete(index);
  if Down then
    inc(index)
  else
    dec(index);
  Item := ListView.Items.Insert(index);
  Item.Data := Data;
  Item.Caption := Str[0];
  Item.SubItems.Add(Str[1]);
  Item.SubItems.Add(Str[2]);
  Item.Selected := true;
  Item.Focused := true;
  btnMoveUp.Visible := Item.Index > 0;
  btnMoveDown.Visible := Item.Index < ListView.Items.Count - 1;
end;

procedure TListForm.Remove;
var
  rownanie: TEquation;
  Item: TListItem;
  index: integer;
begin
  Item := ListView.Selected;
  if Item = nil then
    exit;
  rownanie := Item.Data;
  index := Item.Index;
  ListView.Items.Delete(index);
  EquationList.Remove(rownanie);
  rownanie.Free;
  if index >= ListView.Items.Count then
    dec(index);
  if index >= 0 then
  begin
    Item := ListView.Items[index];
    Item.Selected := true;
    Item.Focused := true;
    ListView.SetFocus;
  end;
end;

procedure TListForm.btnSaveClick(Sender: TObject);
begin
  Save(DatPath);
end;

procedure TListForm.btnRemoveClick(Sender: TObject);
begin
  Remove;
end;

end.
