unit OptionsUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin, ExtCtrls, Recording, Mask;

type
  TOptionsForm = class(TForm)
    cbHideZeros: TCheckBox;
    cbAlsoEps: TCheckBox;
    edEps: TEdit;
    Label4: TLabel;
    Label1: TLabel;
    RadioGroup1: TRadioGroup;
    spinFixed: TSpinEdit;
    spinGeneral: TSpinEdit;
    spinEpsOne: TSpinButton;
    btnClose: TButton;
    btnDefault: TButton;
    cbAutoShiftBack: TCheckBox;
    Label2: TLabel;
    edColumnWidth: TEdit;
    spinInitSize: TSpinEdit;
    Label3: TLabel;
    spinEpsTens: TSpinButton;
    btnResetToIni: TButton;
    ComboBox1: TComboBox;
    Label5: TLabel;
    procedure cbAlsoEpsClick(Sender: TObject);
    procedure edEpsExit(Sender: TObject);
    procedure spinEpsTensDownClick(Sender: TObject);
    procedure spinEpsTensUpClick(Sender: TObject);
    procedure btnDefaultClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbHideZerosClick(Sender: TObject);
    procedure cbAutoShiftBackClick(Sender: TObject);
    procedure edColumnWidthExit(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure spinEpsOneDownClick(Sender: TObject);
    procedure spinEpsOneUpClick(Sender: TObject);
    procedure btnResetToIniClick(Sender: TObject);
    procedure edEpsKeyPress(Sender: TObject; var Key: Char);
    procedure edColumnWidthKeyPress(Sender: TObject; var Key: Char);
    procedure FormActivate(Sender: TObject);
    procedure spinInitSizeChange(Sender: TObject);
    procedure spinInitSizeExit(Sender: TObject);
    procedure spinInitSizeKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure spinGeneralChange(Sender: TObject);
    procedure spinGeneralExit(Sender: TObject);
    procedure spinGeneralKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure spinFixedChange(Sender: TObject);
    procedure spinFixedExit(Sender: TObject);
    procedure spinFixedKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ComboBox1Change(Sender: TObject);
  private
    spinInitSizeKey: Word;
    spinInitSizeSav: integer;
    spinGeneralKey: Word;
    spinGeneralSav: integer;
    spinFixedKey: Word;
    spinFixedSav: integer;
    procedure ControlsToOpt;
    procedure OptToControls;
    procedure RealChangeSpinInitSize;
  public
    procedure SetDefault();
    procedure Apply();
    procedure ApplyWidths();
    procedure ReadFromIni;
    procedure WriteToIni;
  end;

  TOpt = record
    AutoShiftBack: boolean;
    Eps: float;
    HideZero: boolean;
    AlsoEps: boolean;
    FloatFormat: TFloatFormat;
    FixedPrecision: byte;
    GeneralPrecision: byte;
    FormatStr: string[8];
    ColumnsWidth: integer;
    InitSize: integer;
    InitMode: integer;
  end;

var
  OptionsForm: TOptionsForm;
  Opt: TOpt;

function AppDir(): string;

implementation

uses
  Math, IniFiles, GaussUnit;
{$R *.dfm}

function AppDir(): string;
var
  Buffer: array [0 .. MAX_PATH] of Char;
  Len: LongWord;
begin
  Len := GetModuleFileName(0, Buffer, MAX_PATH);
  Buffer[Len] := #0;
  SetString(result, Buffer, Len);
  result := IncludeTrailingPathDelimiter(ExtractFileDir(result));
end;

function IniPath: string;
begin
  result := AppDir() + 'config.ini';
end;

procedure TOptionsForm.ComboBox1Change(Sender: TObject);
begin
  Opt.InitMode := ComboBox1.ItemIndex;
  GaussForm.btnModeAsInit.Enabled := GaussForm.comboMode.ItemIndex+1 <> Opt.InitMode;
end;

procedure TOptionsForm.ControlsToOpt;
begin
  with Opt do
  begin
    AutoShiftBack := cbAutoShiftBack.Checked;
    Eps := StrToFloatDef(edEps.Text, 0);
    HideZero := cbHideZeros.Checked;
    AlsoEps := cbAlsoEps.Checked;
    if RadioGroup1.ItemIndex = 0 then
      FloatFormat := ffFixed
    else
      FloatFormat := ffGeneral;
    FixedPrecision := spinFixed.Value;
    GeneralPrecision := spinGeneral.Value;
    if FloatFormat = ffFixed then
      FormatStr := Format('%%.%df', [FixedPrecision])
    else
      FormatStr := Format('%%.%dg', [GeneralPrecision]);
    ColumnsWidth := StrToIntDef(edColumnWidth.Text, 64);
    InitSize := spinInitSize.Value;
    InitMode := ComboBox1.ItemIndex;
  end;
end;

procedure TOptionsForm.OptToControls;
begin
  cbAutoShiftBack.OnClick := nil;
  cbHideZeros.OnClick := nil;
  cbAlsoEps.OnClick := nil;
  RadioGroup1.OnClick := nil;
  spinFixed.OnChange := nil;
  spinGeneral.OnChange := nil;
  spinInitSize.OnChange := nil;
  ComboBox1.OnChange := nil;
  with Opt do
  begin
    cbAutoShiftBack.Checked := AutoShiftBack;
    edEps.Text := FloatToStr(Eps);
    cbHideZeros.Checked := HideZero;
    cbAlsoEps.Checked := AlsoEps;
    if FloatFormat = ffFixed then
      RadioGroup1.ItemIndex := 0
    else
      RadioGroup1.ItemIndex := 1;
    spinFixed.Value := FixedPrecision;
    spinGeneral.Value := GeneralPrecision;
    edColumnWidth.Text := IntToStr(ColumnsWidth);
    spinInitSize.Value := InitSize;
    ComboBox1.ItemIndex := InitMode;
  end;
  cbAutoShiftBack.OnClick := cbAutoShiftBackClick;
  cbHideZeros.OnClick := cbHideZerosClick;
  cbAlsoEps.OnClick := cbAlsoEpsClick;
  RadioGroup1.OnClick := RadioGroup1Click;
  spinFixed.OnChange := spinFixedChange;
  spinGeneral.OnChange := spinGeneralChange;
  spinInitSize.OnChange := spinInitSizeChange;
  ComboBox1.OnChange := ComboBox1Change;
end;

procedure TOptionsForm.Apply();
begin
  ControlsToOpt();
  GaussForm.ArraysToGrid();
  GaussForm.cbAutoShiftBack.Checked := Opt.AutoShiftBack;
end;

procedure TOptionsForm.ApplyWidths;
var
  i: integer;
begin
  ControlsToOpt();
  with GaussForm.StringGrid do
    for i := 1 to ColCount - 1 do
      ColWidths[i] := Opt.ColumnsWidth;
end;

procedure TOptionsForm.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TOptionsForm.btnDefaultClick(Sender: TObject);
begin
  if (MessageDlg('Do You want to reset to default settigns?', mtWarning,
      [mbYes, mbCancel], 0) = mrYes) then
    SetDefault;
end;

procedure TOptionsForm.btnResetToIniClick(Sender: TObject);
begin
  SetDefault;
  ReadFromIni;
  OptToControls;
  Apply();
end;

procedure TOptionsForm.cbAlsoEpsClick(Sender: TObject);
begin
  if cbAlsoEps.Checked then
    cbHideZeros.Caption := 'Hide <=Eps elements'
  else
    cbHideZeros.Caption := 'Hide zero elements';
  Apply();
end;

procedure TOptionsForm.cbAutoShiftBackClick(Sender: TObject);
begin
  Apply();
end;

procedure TOptionsForm.cbHideZerosClick(Sender: TObject);
begin
  Apply();
end;

procedure TOptionsForm.edEpsExit(Sender: TObject);
begin
  Apply();
end;

procedure TOptionsForm.edEpsKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
  begin
    Apply();
    Key := #0;
  end;
end;

procedure TOptionsForm.edColumnWidthExit(Sender: TObject);
begin
  ApplyWidths();
end;

procedure TOptionsForm.edColumnWidthKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
  begin
    ApplyWidths();
    Key := #0;
  end;
end;

procedure TOptionsForm.FormActivate(Sender: TObject);
begin
  OptToControls;
end;

procedure TOptionsForm.FormCreate(Sender: TObject);
var
  i: integer;
begin
  ComboBox1.Items.AddStrings(GaussForm.comboMode.Items);
  ComboBox1.Items.Insert(0, '--Not specified--');
  ComboBox1.ItemIndex := 4;
  SetDefault;
  ControlsToOpt;
  ReadFromIni;
end;

procedure TOptionsForm.FormDestroy(Sender: TObject);
begin
  WriteToIni;
end;

procedure TOptionsForm.FormHide(Sender: TObject);
begin
  ControlsToOpt();
end;

procedure TOptionsForm.RadioGroup1Click(Sender: TObject);
begin
  Apply();
end;

const
  ModeStrings: array [0 .. 5] of string = ('not_specified', 'simple_mode',
    'exchange_when_GT_zero', 'exchange when_GT_eps', 'partial_pivoting',
    'complete_pivoting');

procedure TOptionsForm.ReadFromIni;
var
  iniFile: TIniFile;
  modeString: string;
  i, n: integer;
begin
  iniFile := TIniFile.Create(IniPath());
  try
    if iniFile.ReadString('Formatting', 'FloatFormat', 'general') = 'fixed' then
      Opt.FloatFormat := ffFixed
    else
      Opt.FloatFormat := ffGeneral;
    Opt.FixedPrecision := iniFile.ReadInteger('Formatting', 'FixedPrecision', Opt.FixedPrecision);
    Opt.GeneralPrecision := iniFile.ReadInteger('Formatting', 'GeneralPrecision', Opt.GeneralPrecision);
    Opt.AutoShiftBack := iniFile.ReadBool('RecordAndPlay', 'AutoShiftBack', Opt.AutoShiftBack);
    modeString := iniFile.ReadString('RecordAndPlay', 'InitMode',  ModeStrings[Opt.InitMode]);
    n := 0;
    for i := 1 to High(ModeStrings) do
      if modeString = ModeStrings[i] then
      begin
        n := i;
        break;
      end;
    Opt.InitMode := n;
    Opt.HideZero := iniFile.ReadBool
      ('Analyze', 'HideZeroElements', Opt.HideZero);
    Opt.AlsoEps := iniFile.ReadBool('Analyze', 'AlsoHideLTEEps', Opt.AlsoEps);
    Opt.ColumnsWidth := iniFile.ReadInteger
      ('Grid', 'ColumnsWidth', Opt.ColumnsWidth);
    Opt.InitSize := iniFile.ReadInteger('OtherSettings', 'InitMatrixSize',
      Opt.InitSize);
    Opt.Eps := iniFile.ReadFloat('OtherSettings', 'Eps', Opt.Eps);
  finally
    iniFile.Free;
  end;
end;

procedure TOptionsForm.WriteToIni;
var
  iniFile: TIniFile;
begin
  iniFile := TIniFile.Create(IniPath());
  try
    if Opt.FloatFormat = ffFixed then
      iniFile.WriteString('Formatting', 'FloatFormat', 'fixed')
    else
      iniFile.WriteString('Formatting', 'FloatFormat', 'general');
    iniFile.WriteInteger('Formatting', 'FixedPrecision', Opt.FixedPrecision);
    iniFile.WriteInteger('Formatting', 'GeneralPrecision',
      Opt.GeneralPrecision);
    iniFile.WriteBool('RecordAndPlay', 'AutoShiftBack', Opt.AutoShiftBack);
    if Opt.InitMode >= 0 then
      iniFile.WriteString('RecordAndPlay', 'InitMode', ModeStrings[Opt.InitMode]);
    iniFile.WriteBool('Analyze', 'HideZeroElements', Opt.HideZero);
    iniFile.WriteBool('Analyze', 'AlsoHideLTEEps', Opt.AlsoEps);
    iniFile.WriteInteger('Grid', 'ColumnsWidth', Opt.ColumnsWidth);
    iniFile.WriteInteger('OtherSettings', 'InitMatrixSize', Opt.InitSize);
    iniFile.WriteFloat('OtherSettings', 'Eps', Opt.Eps);
  finally
    iniFile.Free;
  end;
end;

procedure TOptionsForm.SetDefault;
begin
  edEps.Text := '1e-12';
  cbHideZeros.Checked := true;
  cbAlsoEps.Checked := true;
  spinFixed.Value := 4;
  spinGeneral.Value := 5;
  RadioGroup1.ItemIndex := 1;
  cbAutoShiftBack.Checked := true;
  edColumnWidth.Text := '64';
  spinInitSize.Value := 3;
end;

function TruncDown(f: double): int64;
begin
  result := Trunc(f);
  if (result <> f) and (f < 0) then
    result := result - 1;
end;

procedure TOptionsForm.spinEpsOneDownClick(Sender: TObject);
var
  d, dlog: float;
  Characteristic: integer;
  Mantissa: float;
begin
  d := StrToFloat(edEps.Text);
  if d = 0 then
    exit;
  if d <= 1E-19 then
    d := 0
  else
  begin
    dlog := Log10(d);
    Characteristic := TruncDown(dlog);
    Mantissa := Power(10, dlog - Characteristic);
    if Mantissa >= 2 then
      Mantissa := Mantissa - 1
    else
      Mantissa := Mantissa - 0.1;
    d := Mantissa * Power(10, Characteristic);
  end;
  edEps.Text := FloatToStr(d);
end;

procedure TOptionsForm.spinEpsOneUpClick(Sender: TObject);
var
  d, dlog: float;
  Characteristic: integer;
  Mantissa: float;
begin
  d := StrToFloat(edEps.Text);
  if d = 0 then
  begin
    edEps.Text := '1E-19';
    exit;
  end;
  dlog := Log10(d);
  Characteristic := TruncDown(dlog);
  Mantissa := Power(10, dlog - Characteristic);
  Mantissa := Mantissa + 1;
  d := Mantissa * Power(10, Characteristic);
  edEps.Text := FloatToStr(d);
end;

procedure TOptionsForm.spinEpsTensDownClick(Sender: TObject);
var
  d: float;
begin
  d := StrToFloat(edEps.Text);
  d := d / 10;
  if d < 1E-19 then
    d := 0;
  edEps.Text := FloatToStr(d);
  Apply();
end;

procedure TOptionsForm.spinEpsTensUpClick(Sender: TObject);
var
  d: float;
begin
  d := StrToFloat(edEps.Text);
  if d = 0 then
    d := 1E-19
  else
    d := d * 10;
  edEps.Text := FloatToStr(d);
  Apply();
end;

procedure TOptionsForm.spinFixedChange(Sender: TObject);
var
  n: integer;
begin
  if not TryStrToInt(spinFixed.Text, n) then
    exit;
  if spinFixedKey = 0 then
    Apply();
  spinFixedSav := spinFixed.Value;
  spinFixedKey := 0;
end;

procedure TOptionsForm.spinFixedExit(Sender: TObject);
begin
  Apply();
end;

procedure TOptionsForm.spinFixedKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = 13 then
  begin
    Key := 0;
    Apply();
    spinFixedKey := 0;
  end
  else
    spinFixedKey := Key;
end;

procedure TOptionsForm.spinGeneralChange(Sender: TObject);
var
  n: integer;
begin
  if not TryStrToInt(spinGeneral.Text, n) then
    exit;
  if spinGeneralKey = 0 then
    Apply();
  spinGeneralSav := spinGeneral.Value;
  spinGeneralKey := 0;
end;

procedure TOptionsForm.spinGeneralExit(Sender: TObject);
begin
  Apply();
end;

procedure TOptionsForm.spinGeneralKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = 13 then
  begin
    Key := 0;
    Apply();
    spinGeneralKey := 0;
  end
  else
    spinGeneralKey := Key;
end;

procedure TOptionsForm.spinInitSizeChange(Sender: TObject);
var
  n: integer;
begin
  if not TryStrToInt(spinInitSize.Text, n) then
    exit;
  if (spinInitSizeKey = 0) and (Abs(spinInitSizeSav - spinInitSize.Value) = 1)
    then
    RealChangeSpinInitSize;
  spinInitSizeSav := spinInitSize.Value;
  spinInitSizeKey := 0;
end;

procedure TOptionsForm.spinInitSizeExit(Sender: TObject);
begin
  RealChangeSpinInitSize
end;

procedure TOptionsForm.spinInitSizeKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = 13 then
  begin
    Key := 0;
    RealChangeSpinInitSize;
    spinInitSizeKey := 0;
  end
  else
    spinInitSizeKey := Key;
end;

procedure TOptionsForm.RealChangeSpinInitSize;
begin
  Opt.InitSize := spinInitSize.Value;
  GaussForm.btnNumberEqAsInit.Enabled := GaussForm.spinSize.Value <> Opt.InitSize;
end;

end.
